@@ -10,13 +10,13 @@
 #
 # $Project: /Convert-Binary-C $
 # $Author: mhx $
-# $Date: 2006/03/11 18:26:20 +0100 $
-# $Revision: 30 $
+# $Date: 2007/06/11 20:59:50 +0200 $
+# $Revision: 32 $
 # $Source: /lib/Convert/Binary/C/Cached.pm $
 #
 ################################################################################
 #
-# Copyright (c) 2002-2006 Marcus Holland-Moritz. All rights reserved.
+# Copyright (c) 2002-2007 Marcus Holland-Moritz. All rights reserved.
 # This program is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself.
 #
@@ -31,7 +31,7 @@ use vars qw( @ISA $VERSION );
 
 @ISA = qw(Convert::Binary::C);
 
-$VERSION = do { my @r = '$Snapshot: /Convert-Binary-C/0.64_02 $' =~ /(\d+\.\d+(?:_\d+)?)/; @r ? $r[0] : '9.99' };
+$VERSION = do { my @r = '$Snapshot: /Convert-Binary-C/0.69_01 $' =~ /(\d+\.\d+(?:_\d+)?)/; @r ? $r[0] : '9.99' };
 $VERSION = eval $VERSION;
 
 sub new
@@ -372,15 +372,18 @@ Convert::Binary::C::Cached - Caching for Convert::Binary::C
   #------------------------
   $c = Convert::Binary::C::Cached->new(
          Cache   => '/tmp/cache.c',
-         Include => ['include']
+         Include => [
+           '/usr/lib/gcc-lib/i686-pc-linux-gnu/3.3.6/include',
+           '/usr/include',
+         ],
        );
   
-  #-------------------------------------------------
-  # Parse 'stdio.h' and dump the definition of FILE
-  #-------------------------------------------------
-  $c->parse_file('stdio.h');
+  #----------------------------------------------------
+  # Parse 'time.h' and dump the definition of timespec
+  #----------------------------------------------------
+  $c->parse_file('time.h');
   
-  print Dumper($c->typedef('FILE'));
+  print Dumper($c->struct('timespec'));
 
 =head1 DESCRIPTION
 
@@ -441,7 +444,7 @@ algorithm cannot detect that an update is required.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2002-2006 Marcus Holland-Moritz. All rights reserved.
+Copyright (c) 2002-2007 Marcus Holland-Moritz. All rights reserved.
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
